#-*- Makefile -*-

require DE_EPAGES::Core::API::Makefile;
__DATA__

!include host-$(HOSTNAME).mk

PATCH_TAG	= patch_6_17_5
PATCH_VERSION	= 6.17.5
DE_EPAGES_DIR	= $(EPAGES_CARTRIDGES)\DE_EPAGES
DATA_DIR	= $(EPAGES_SHARED)\Patches\var-$(PATCH_TAG)
LOG_DIR		= $(DATA_DIR)\logs\$(HOSTNAME)
TGT		= $(DATA_DIR)\tgt\$(HOSTNAME)

CREATE_PATCHDB	= $(PERL) Scripts/createPatchDB.pl
DROP_PATCHDB	= $(PERL) Scripts/dropPatchDB.pl
CLEAN_PATCHDB   = $(PERL) Scripts/cleanPatchDB.pl

PATCH_SERVICE	=
PATCH_REQROUTER	=
PATCH_MONITOR	=
STOP_EPAGES	=
START_EPAGES	=
RESTART_EPAGES	= 1

!OS_SPECIFIC_DEFINES
PRL		= $(PERL) -IInclude
INCLUDES	= -I$(EPAGES_CARTRIDGES)/.version/$(PATCH_VERSION)
!ifdef WINDOWS
PATCH_WEBADAPTER= 1
IF_EXIST	= if exist
IF_NOT_EXIST	= if not exist
DO		=
RMDIR		= rmdir /S /Q
DIFF		= gdiff
MAKE		= nmake -nologo
!else
PATCH_WEBADAPTER=
SHELL		= /bin/bash
IF_EXIST	= [ ! -e
IF_NOT_EXIST	= [ -e
DO		= ] ||
RMDIR		= rm -rf
DIFF		= diff
EPAGES_PROJECT	?= epages6
MAKE		= make
PKG		= $(shell test -f /etc/debian_version && echo deb || echo rpm)
INI_CONFIG	= /var/epages/ini-config.sh
INSTALL_RPM	= /var/epages/install-RPM.sh
!endif

!ifndef EPAGES_REPO
EPAGES_REPO = http://www.epages.com/repo/epages
!endif

LOOP_STORES	= $(PRL) Scripts/LoopStores.pl
START_SERVICE   = $(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/startService.pl -ifexists
STOP_SERVICE    = $(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/stopService.pl -ifexists
CREATE_CTMPLS   = $(PRL) $(DE_EPAGES_DIR)/Presentation/Scripts/createCtmpls.pl

CALL_PATCH_HANDLER = $(PERL) Scripts/call-PatchHandler.pl
SET_PATCHDB_BASIC_ATTRIBUTES = $(PERL) Scripts/setPatchDBBasicAttributes.pl
INITIALIZE_PATCH_SYSTEM = $(PERL) -IIncludeHandler Scripts/initializePatchSystemInfoDB.pl

STORE		= dummy

.force:

# we always need following target for patch.pl:
get_var: .force
	@echo LOG_DIR=$(LOG_DIR)
	@echo TGT_DIR=$(DATA_DIR)\tgt
	@echo TGT=$(TGT)

# how to install the epages patch (see install_en.txt)
# ****************************************************
#-------------------------------------------------------------------------------------------
#    | execute                             | on  UNIX                  | on Windows        |
#-------------------------------------------------------------------------------------------
# 1. | set environment                     | . /etc/default/epages6    |                   |
# 2. | create makefile (needs epages perl) | perl Makefile.PL          | perl Makefile.PL  |
# 3. | pre-installation tasks              | make preinstall           | nmake preinstall  |
# 4. | set environment                     | . /etc/default/epages6    |                   |
# 5. | installation tasks* (see NOTE)      | make install              | nmake install     |
#-------------------------------------------------------------------------------------------
# *) if epages runs on more than 1 computer, execute this step only on your main application server

create_patch_system_info_db:
	$(CREATE_PATCHDB)

drop_patch_system_info_db:
	$(DROP_PATCHDB)

clean_patch_system_info_db:
	$(CLEAN_PATCHDB)

reinitialize:
	$(MAKE) clean_patch_system_info_db
	$(MAKE) create_patch_system_info_db
	$(DEL) $(DATA_DIR)\tgt\initialize
	$(MAKE) initialize

initialize: $(DATA_DIR)\tgt\initialize
$(DATA_DIR)\tgt\initialize:
	$(MAKE) clean_patch_system_info_db
!ifndef WINDOWS
	./Scripts/install-hotfixes.sh -stop $(PATCH_VERSION)
!endif
	$(MAKE) create_patch_system_info_db
	$(INITIALIZE_PATCH_SYSTEM) -patchsystemdbsourcedir Database/mysql
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)

download: $(DATA_DIR)\tgt\initialize
	$(CALL_PATCH_HANDLER) -handler PatchDownloadHandler

prepare: $(DATA_DIR)\tgt\initialize $(DATA_DIR)\tgt\prepare
$(DATA_DIR)\tgt\prepare:
	$(MAKE) prepare_update_os
	$(CALL_PATCH_HANDLER) -handler PatchDownloadHandler
	$(CALL_PATCH_HANDLER) -handler PatchPrepareHandler
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)
!ifndef WINDOWS
	$(INI_CONFIG) -f prerequisits.ini -s EPAGES -k notests -v WebServicePortScan -i
!endif

preinstall: prepare $(TGT)\preinstall
$(TGT)\preinstall:
!ifndef WINDOWS
	$(MAKE) prepare_update_os
	./Scripts/install-hotfixes.sh -stop $(PATCH_VERSION)
!endif
	$(MAKE) stop_epages update_os httpd\close stop_cron
	$(DEL) $(DATA_DIR)\tgt\posthosts.txt
	$(PRL) Scripts/checkScheduler.pl -hostname $(HOSTNAME)
	$(AWK) "/^$(HOSTNAME)$$/{i=1}END{exit i!=1}" $(DATA_DIR)\tgt\prehosts.txt 2>$(NUL) || echo $(HOSTNAME)>> $(DATA_DIR)\tgt\prehosts.txt
	$(MAKE) touch DIR=$(@D) ADDF=$(@F)

install: preinstall close_stores restart_epages alpha_update_os
	$(CALL_PATCH_HANDLER) -handler PatchInstallHandler
	$(MAKE) update_cartridge_dir update_ctmpls
	echo $(HOSTNAME)> $(DATA_DIR)\tgt\alphahost.txt
	$(MAKE) touch DIR=$(DATA_DIR)\tgt ADDF=install
	$(MAKE) postinstall IS_ALPHA=1

cleanup:
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerCleanUp

recreate_solr_search_index:
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerRemoteSearch

set_patchdb_basic_attributes:
	$(SET_PATCHDB_BASIC_ATTRIBUTES) -key $(KEY) -value $(VALUE)

export_multistore_master:
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerMultiStore

postinstall:
!ifndef IS_ALPHA
	@$(IF_NOT_EXIST) $(DATA_DIR)\tgt\$@-alpha $(DO) echo "Postinstall on `hostname` failed: First postinstall alpha server"
	@$(IF_NOT_EXIST) $(DATA_DIR)\tgt\$@-alpha $(DO) $(PERL) -e "exit 1;"
!endif
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandler
!ifndef WINDOWS
	/var/epages/movelink.sh /etc/default/$(EPAGES_PROJECT).lnk
	/var/epages/movelink.sh /etc/init.d/$(EPAGES_PROJECT).lnk
!endif
	$(MAKE) perm start_cron restart_httpd
!ifdef IS_ALPHA
	echo $(HOSTNAME)>> $(DATA_DIR)\tgt\$@-alpha
#       if single server installation go on with reopen
	$(PERL) -MFile::Compare -e "exit not compare('$(DATA_DIR)\tgt\alphahost.txt','$(DATA_DIR)\tgt\prehosts.txt')"\
 || $(MAKE) reopen IS_ALPHA=1
!else
	$(MAKE) start_epages
!endif

reopen:
!ifdef IS_ALPHA
	$(MAKE) open_stores
!ifndef NO_RESTART
	$(MAKE) restart_epages
!endif
!ifndef FLEX_INSTALLATION
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerSynchronizeSite
!endif
	$(CALL_PATCH_HANDLER) -handler PatchPostInstallHandlerRemoveCartridges
	$(PRL) Scripts/findOverloadedTemplates.pl -store all -patchdir . -initial $(INITIAL_VERSION)
	$(MAKE) export_multistore_master
!endif

# targets (patch database)
# ************************

close_stores: loop_stores.mak
	$(MAKE) make_makefile
!ifdef EPAGES_DATABASE_SITE
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) close_store STORE=Site STORE_TYPE=Site
!endif
	$(MAKE) -f loop_stores.mak close_stores TGT="$(TGT)"
close_store:
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) close_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE)
	$(DEL) $(TGT)\open_store\$(STORE)

open_stores: loop_stores.mak
	$(MAKE) make_makefile
!ifdef EPAGES_DATABASE_SITE
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) open_store STORE=Site STORE_TYPE=Site
!endif
	$(MAKE) -f loop_stores.mak open_stores TGT="$(TGT)"
open_store:
	$(CD) $(DE_EPAGES_DIR) && $(MAKE) open_store STORE=$(STORE) STORE_TYPE=$(STORE_TYPE)
	$(DEL) $(TGT)\close_store\$(STORE)

restore_styles: loop_stores.mak
	$(MAKE) -f loop_stores.mak restore_styles TGT="$(TGT)"
restore_style:
	$(PRL) $(DE_EPAGES_DIR)/Design/Scripts/invalidateAllStyles.pl -storename $(STORE)

update_ctmpls: loop_stores.mak
!ifdef EPAGES_DATABASE_SITE
!ifdef CTMPL_TXT
	$(CREATE_CTMPLS) -storename Site -listfile ctmpl-Site.txt -nooutput
!else
	$(CREATE_CTMPLS) -storename Site -delete -nooutput
!endif
!ifdef DEL_SAVED_INCLUDES
	$(IF_EXIST) $(EPAGES_STATIC)\Site\SavedIncludes $(DO) $(RMDIR) $(EPAGES_STATIC)\Site\SavedIncludes
!endif
!endif
	$(MAKE) -f loop_stores.mak update_ctmpls TGT="$(TGT)"
update_ctmpl:
!ifdef CTMPL_TXT
	$(CREATE_CTMPLS) -storename $(STORE) -listfile ctmpl-$(STORE).txt -nooutput
!else
	$(CREATE_CTMPLS) -storename $(STORE) -delete -nooutput
!endif
!ifdef DEL_SAVED_INCLUDES
	$(IF_EXIST) $(EPAGES_STATIC)\$(STORE)\SavedIncludes $(DO) $(RMDIR) $(EPAGES_STATIC)\$(STORE)\SavedIncludes
!endif

test_styles: loop_stores.mak
	$(MAKE) -f loop_stores.mak test_styles TGT="$(TGT)"
test_style:
	$(PRL) $(DE_EPAGES_DIR)/Installer/Scripts/compareCartridgeData.pl -storename $(STORE) -type $(STORE_TYPE)

make_makefile:
	$(CD) $(DE_EPAGES_DIR) && $(PRL) Makefile.PL

loop_stores.mak:
	$(LOOP_STORES) -mak makefile> $@.out
	$(MOVE) $@.out $@

update_cartridge_dir:
!ifndef WINDOWS
!BEGIN_VERBATIM
	chown -R ep_appl:epages "$(EPAGES_CARTRIDGES)/"
	find "$(EPAGES_CARTRIDGES)" -type f -name '*.??.xml' -exec chmod 644 {} \;
	find "$(EPAGES_CARTRIDGES)" -type f -name '*.sh' -exec chmod 755 {} \;
	chmod 644 "$(EPAGES_CARTRIDGES)"/*/*/Data/Public/BO/*.??.dot
!END_VERBATIM
!endif

# auxiliary targets
# *****************

stop_cron:
	$(IF_NOT_EXIST) "$(DATA_DIR)"\tgt\prehosts.txt $(DO) $(PRL) Scripts/EditSchedulerConf.pl -stop $(CRON_FILES) -delete "FactFinderInit FactFinderExportProducts FactFinderExportProductsTriggered"

start_cron:
	$(PRL) Scripts/EditSchedulerConf.pl -start $(CRON_FILES)
	$(START_SERVICE) cron

stop_epages: .force
!ifeq ("$(STOP_EPAGES)", "1")
	$(STOP_SERVICE) epages
!ifndef WINDOWS
	if pgrep RequestRouter ; then $(ETC_INITD) stop_rr || pkill -9 RequestRouter ; fi
!endif
!endif

start_epages: .force
!ifeq ("$(START_EPAGES)", "1")
	$(START_SERVICE) epages
!endif

restart_epages:
!ifeq ("$(RESTART_EPAGES)", "1")
!ifdef IS_ALPHA
	$(PRL) $(DE_EPAGES_DIR)/WebInterface/Scripts/shutdownAppSrv.pl -all -sleep 0 || $(ECHONL)
!endif
!endif

httpd\close httpd\open:
!ifndef WINDOWS
	$(ETC_INITD) start_httpd $(@F)
!endif

restart_httpd:
!ifndef WINDOWS
	$(MAKE) httpd\open
!endif

alpha_update_os:
!ifndef WINDOWS
	ENABLE_REPO=epages-update $(INSTALL_RPM) --update-only epages-j
	ENABLE_REPO=epages-update $(INSTALL_RPM) --update-only epages-hotfix-base

!endif

prepare_update_os:
!ifndef WINDOWS
!ifeq ("$(PKG)", "rpm")
	rpm -q --whatprovides 'epages-release(6.17.5)' || { \
 curl -O $(EPAGES_REPO)/rpmify/rpmify-epages.sh ; bash rpmify-epages.sh -repo $(EPAGES_REPO); }
!endif
	ENABLE_REPO=epages-base $(INSTALL_RPM) --update epages-requires
	ENABLE_REPO=epages-base $(INSTALL_RPM) --update-only epages-webconf
!ifndef DEBIAN
	rpm --quiet -q epages-websrv && ENABLE_REPO=epages-base $(INSTALL_RPM) --update epages-webconf || $(ECHONL)
!endif
	ENABLE_REPO=epages-update $(INSTALL_RPM) --update epages-nodejs
!endif

update_os: prepare_update_os
!ifndef WINDOWS
	/var/epages/movelink.sh /etc/default/$(EPAGES_PROJECT)
	/var/epages/movelink.sh /etc/init.d/$(EPAGES_PROJECT)
	for i in `ls sh/*/EPAGES_RELEASE/var/epages/patch-sh.sh 2>/dev/null` ; do\
 cat $$i > /var/epages/patch-sh.sh ; done
	/var/epages/patch-sh.sh -d sh -i $(INITIAL_INTERNAL_VERSION)
!endif

restore_os:
!ifndef WINDOWS
	/var/epages/patch-sh.sh -d sh -R -i $(INITIAL_INTERNAL_VERSION)
!endif

perm:
!ifndef WINDOWS
	-$(ETC_INITD) perm admin
	$(IF_EXIST) $(EPAGES_PERL)/bin $(DO) chmod 755 $(EPAGES_PERL)/bin/*
!endif

touch:
	$(MKDIR) $(DIR)
	$(ECHONL) > $(DIR)\$(ADDF)

del_backup_files:
	$(IF_EXIST) "$(DATA_DIR)\tgt" $(DO) $(RMDIR) "$(DATA_DIR)\tgt"
	$(IF_EXIST) "$(DATA_DIR)\Config" $(DO) $(RMDIR) "$(DATA_DIR)\Config"
	$(IF_EXIST) "$(DATA_DIR)\DE_EPAGES" $(DO) $(RMDIR) "$(DATA_DIR)\DE_EPAGES"
	$(DEL) "$(DATA_DIR)\Cartridges.xml"
	$(PRL) Scripts/DeleteOldStoreTypeDirs.pl -current $(PATCH_VERSION)
	$(IF_EXIST) "$(EPAGES_CARTRIDGES)\.version" $(DO) $(RMDIR) "$(EPAGES_CARTRIDGES)\.version"
	$(IF_EXIST) "$(EPAGES_CARTRIDGES)\.lang" $(DO) $(RMDIR) "$(EPAGES_CARTRIDGES)\.lang"

transform_text2string_attribute:
	$(PRL) _CARTRIDGES_/DE_EPAGES/Object/Scripts/transformTextattributeToStringAttributePatch_6_13_0.pl -store all

#-*- Makefile -*-
